﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

using Arction.WinForms.Charting;
using Arction.WinForms.Charting.Axes;
using Arction.WinForms.Charting.SeriesXY;
using Arction.WinForms.Charting.Views.ViewXY;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 차트
        /// </summary>
        private LightningChartUltimate chart = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            InitializeChart();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 뷰 XY 확대/축소시 처리하기 - ViewXY_Zoomed(sender, e)

        /// <summary>
        /// 뷰 XY 확대/축소시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ViewXY_Zoomed(object sender, ZoomedXYEventArgs e)
        {
            ViewXY viewXY = sender as ViewXY;
            AxisX axisX   = viewXY.XAxes[0];

            double minimum = 0;
            double maximum = 1000;

            double currentMinimum = axisX.Minimum;
            double currentMaximum = axisX.Maximum;

            if(minimum > currentMinimum || maximum < currentMaximum)
            {
                if(minimum > currentMinimum)
                {
                    currentMinimum = minimum;
                }

                if(maximum < currentMaximum)
                {
                    currentMaximum = maximum;
                }

                viewXY.OwnerChart.BeginUpdate();

                axisX.SetRange(currentMinimum, currentMaximum);

                viewXY.OwnerChart.EndUpdate();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 차트 포인트 리스트 구하기 - GetChartPointList(random, pointCount, minimumAmplitude, maximumAmplitude)

        /// <summary>
        /// 차트 포인트 리스트 구하기
        /// </summary>
        /// <param name="random">난수기</param>
        /// <param name="pointCount">포인트 수</param>
        /// <param name="minimumAmplitude">최소 크기</param>
        /// <param name="maximumAmplitude">최대 크기</param>
        /// <returns>차트 포인트 리스트</returns>
        private List<ChartPoint> GetChartPointList(Random random, int pointCount, double minimumAmplitude, double maximumAmplitude)
        {
            double stepX = 1000 / (pointCount - 1);

            double previousY = minimumAmplitude;

            List<ChartPoint> list = new List<ChartPoint>();

            for(int i = 0; i < pointCount; i++)
            {
                ChartPoint point = new ChartPoint();

                point.X = (double)i * stepX;

                double y = previousY + (random.NextDouble() - 0.5) * maximumAmplitude / 5;

                if(y < minimumAmplitude / 2)
                {
                    y = minimumAmplitude;
                }

                if(y > maximumAmplitude * 1.5)
                {
                    y = maximumAmplitude;
                }

                point.Y = y;

                list.Add(point);

                previousY = y;
            }

            return list;
        }

        #endregion
        #region 시리즈 데이터 리스트 구하기 - GetSeriesDataList(pointCount)

        /// <summary>
        /// 시리즈 데이터 리스트 구하기
        /// </summary>
        /// <param name="pointCount">포인트 수</param>
        /// <returns>시리즈 데이터 리스트</returns>
        private List<SeriesData> GetSeriesDataList(int pointCount)
        {
            Random random = new Random();

            List<SeriesData> list = new List<SeriesData>();

            SeriesData data;

            data = new SeriesData();

            data.ChartPointList = GetChartPointList(random, pointCount, 25000.0, 50000.0);

            list.Add(data);

            data = new SeriesData();

            data.ChartPointList = GetChartPointList(random, pointCount, 18000.0, 30000.0);

            list.Add(data);

            data = new SeriesData();

            data.ChartPointList = GetChartPointList(random, pointCount, 5000.0, 15000.0);

            list.Add(data);

            return list;
        }

        #endregion
        #region 차트 데이터 설정하기 - SetChartData(viewXY, axisX, axisY, seriesDataList)

        /// <summary>
        /// 차트 데이터 설정하기
        /// </summary>
        /// <param name="viewXY">뷰 XY</param>
        /// <param name="axisX">X축</param>
        /// <param name="axisY">Y축</param>
        /// <param name="seriesDataList">시리즈 데이터 리스트</param>
        private void SetChartData(ViewXY viewXY, AxisX axisX, AxisY axisY, List<SeriesData> seriesDataList)
        {
            HighLowSeriesPoint[] previousPointArray = null;

            for(int i = 2; i >= 0; i--)
            {
                HighLowSeries series = new HighLowSeries(viewXY, axisX, axisY);

                series.Fill.Color          = ChartHelper.TranslucentColorList[i];
                series.Fill.GradientFill   = GradientFill.Solid;
                series.LineStyle.Width     = 2f;
                series.LineStyleHigh.Color = ChartHelper.SolidColorList[4];
                series.LineVisibleLow      = false;
                series.MouseHighlight      = MouseOverHighlight.None;
                series.MouseInteraction    = false;
                series.Title.Text          = "시리즈 " + (3 - i).ToString();

                SeriesData seriesData = seriesDataList[i];

                List<ChartPoint> pointList = seriesData.ChartPointList;

                HighLowSeriesPoint[] pointArray = new HighLowSeriesPoint[pointList.Count];

                for(int j = 0; j < pointList.Count; j++)
                {
                    pointArray[j].X     = pointList[j].X;
                    pointArray[j].YLow  = 0;
                    pointArray[j].YHigh = pointList[j].Y;

                    if(i < 2)
                    {
                        pointArray[j].YLow  += previousPointArray[j].YHigh;
                        pointArray[j].YHigh += previousPointArray[j].YHigh;
                    }
                }

                series.Points = pointArray;

                viewXY.HighLowSeries.Add(series);

                previousPointArray = pointArray;
            }
        }

        #endregion
        #region 차트 초기화하기 - InitializeChart()

        /// <summary>
        /// 차트 초기화하기
        /// </summary>
        private void InitializeChart()
        {
            this.chart = new LightningChartUltimate();
            
            this.chart.BeginUpdate();
            
            this.chart.Name   = "chart";
            this.chart.Parent = this;
            this.chart.Dock   = DockStyle.Fill;

            this.chart.Title.Font = Font;
            this.chart.Title.Text = "스택 영역 차트";

            ViewXY viewXY = this.chart.ViewXY;

            viewXY.ZoomPanOptions.MouseWheelZooming = MouseWheelZooming.Horizontal;
            viewXY.ZoomPanOptions.PanDirection      = PanDirection.Horizontal;

            viewXY.Zoomed += ViewXY_Zoomed;
            
            AxisX axisX = this.chart.ViewXY.XAxes[0];

            axisX.Title.Font = Font;
            axisX.Title.Text = "X축";
            axisX.ValueType  = AxisValueType.Number;
            axisX.ScrollMode = XAxisScrollMode.None;

            AxisY axisY = this.chart.ViewXY.YAxes[0];

            axisY.Title.Font     = Font;
            axisY.Title.Text     = "Y축";
            axisY.PanningEnabled = false;

            ChartHelper.SetDarkFlatStyle(this.chart);

            List<SeriesData> seriesDataList = GetSeriesDataList(500);

            SetChartData(viewXY, axisX, axisY, seriesDataList);

            viewXY.ZoomToFit();

            LegendBoxXY legendBoxXY = this.chart.ViewXY.LegendBoxes[0];

            legendBoxXY.Position = LegendBoxPositionXY.SegmentBottomRight;

            this.chart.EndUpdate();
        }

        #endregion
    }
}